@@ -27,6 +27,7 @@ module Data.Text.Zipper
 
     -- * Navigation functions
     , moveCursor
+    , moveCursorClosest
     , moveRight
     , moveLeft
     , moveUp
@@ -58,6 +59,7 @@ where
 import Control.Applicative ((<$>))
 import Control.DeepSeq
 import Data.Char (isPrint)
+import Data.List (foldl')
 import Data.Monoid
 import qualified Data.Text as T
 import qualified Data.Vector as V
@@ -145,7 +147,10 @@ mkZipper fromCh drp tk lngth lst int nl linesFunc toListF ls lmt =
         (first, rest) = if null limitedLs
                         then (mempty, mempty)
                         else (head limitedLs, tail limitedLs)
-    in TZ mempty first [] rest fromCh drp tk lngth lst int nl linesFunc toListF lmt
+        numLines = length ls
+        insertLine z (i, l) = (if i < numLines - 1 then breakLine else id) $ insertMany l z
+        loadInitial z = foldl' insertLine z $ zip [0..] (first:rest)
+    in loadInitial $ TZ mempty mempty mempty mempty fromCh drp tk lngth lst int nl linesFunc toListF lmt
 
 -- | Get the text contents of the zipper.
 getText :: (Monoid a) => TextZipper a -> [a]
@@ -185,6 +190,22 @@ moveCursor (row, col) tz =
                , toRight = drop_ tz col (t !! row)
                }
 
+-- | Move the cursor to the specified row and column. Invalid cursor
+-- positions will be reinterpreted as the closest valid position. Valid
+-- cursor positions range as described for 'cursorPosition'.
+moveCursorClosest :: (Monoid a) => (Int, Int) -> TextZipper a -> TextZipper a
+moveCursorClosest (row, col) tz =
+    let t = getText tz
+        bestRow = min (max 0 $ length t - 1) $ max 0 row
+        bestCol = if bestRow < length t
+                  then min (length_ tz (t !! bestRow)) $ max 0 col
+                  else 0
+    in tz { above = take bestRow t
+          , below = drop (bestRow + 1) t
+          , toLeft = take_ tz bestCol (t !! bestRow)
+          , toRight = drop_ tz bestCol (t !! bestRow)
+          }
+
 isFirstLine :: TextZipper a -> Bool
 isFirstLine = null . above
 
@@ -207,28 +228,19 @@ currentLine tz = (toLeft tz) `mappend` (toRight tz)
 -- Otherwise insert the character and move the cursor one position to
 -- the right.
 insertChar :: (Monoid a) => Char -> TextZipper a -> TextZipper a
-insertChar ch tz = maybe tz id $ insertChar_ ch tz
-
-insertChar_ :: (Monoid a) => Char -> TextZipper a -> Maybe (TextZipper a)
-insertChar_ ch tz
-    | ch == '\n' = breakLine_ tz
-    | isPrint ch = Just $ tz { toLeft = toLeft tz `mappend` (fromChar tz ch) }
-    | otherwise  = Nothing
+insertChar ch tz
+    | ch == '\n' = breakLine tz
+    | isPrint ch = tz { toLeft = toLeft tz `mappend` (fromChar tz ch) }
+    | otherwise  = tz
 
 -- | Insert many characters at the current cursor position. Move the
 -- cursor to the end of the inserted text.
 insertMany :: (Monoid a) => a -> TextZipper a -> TextZipper a
-insertMany str tz =
-    let go [] z = z
-        go (c:cs) z = maybe z (go cs) $ insertChar_ c z
-    in go (toList_ tz str) tz
+insertMany str tz = foldl' (flip insertChar) tz $ toList_ tz str
 
 -- | Insert a line break at the current cursor position.
 breakLine :: (Monoid a) => TextZipper a -> TextZipper a
-breakLine tz = maybe tz id $ breakLine_ tz
-
-breakLine_ :: (Monoid a) => TextZipper a -> Maybe (TextZipper a)
-breakLine_ tz =
+breakLine tz =
     -- Plus two because we count the current line and the line we are
     -- about to create; if that number of lines exceeds the limit,
     -- ignore this operation.
@@ -237,9 +249,9 @@ breakLine_ tz =
                       }
     in case lineLimit tz of
           Just lim -> if length (above tz) + length (below tz) + 2 > lim
-                      then Nothing
-                      else Just modified
-          Nothing -> Just modified
+                      then tz
+                      else modified
+          Nothing -> modified
 
 -- | Move the cursor to the end of the current line.
 gotoEOL :: (Monoid a) => TextZipper a -> TextZipper a
