@@ -3,8 +3,6 @@
 {-# OPTIONS_GHC -fno-warn-orphans #-}
 module WordsSpec (spec) where
 
--- This test suite is Copyright (c) Hans-Peter Deifel
-
 import Test.Hspec
 import Test.QuickCheck
 
@@ -17,11 +15,31 @@ import Data.Text.Zipper.Generic.Words
 
 spec :: Spec
 spec = do
+    constructorSpec
+    insertCharSpec
+    insertManySpec
     moveWordLeftSpec
     moveWordRightSpec
     deletePrevWordSpec
     deleteWordSpec
 
+constructorSpec :: Spec
+constructorSpec = describe "constructor" $ do
+    it "inserts only printable characters at construction time" $
+      (stringZipper ["abc\x1b def"] Nothing) `shouldBe` (stringZipper ["abc def"] Nothing)
+
+insertCharSpec :: Spec
+insertCharSpec = describe "insertChar" $ do
+    it "ignores an insert of a non-printable character" $
+      let z = stringZipper [] Nothing
+      in (insertChar '\x1b' z) `shouldBe` z
+
+insertManySpec :: Spec
+insertManySpec = describe "insertMany" $ do
+    it "ignores an insert of a non-printable character" $
+      let z = stringZipper ["abc"] Nothing
+      in (insertMany "ghi\x1bjkl" z) `shouldBe` (insertMany "ghijkl" z)
+
 moveWordLeftSpec :: Spec
 moveWordLeftSpec = describe "moveWordLeft" $ do
     it "does nothing at the start of the text" $
